  /*
   * Blinky 2 (c) Alisofin
   * Plays songs and tales
   * Button Power: when pushed, system is started or shut down
   * Button Folder: when turned, folder is changed. When pushed, the next file is the current folder is played
   * Button Volume: when turned, volume is changed. When pushed, pause/unpause the current played file.
   * System automatically shuts down when no interaction for a certain time
   * 
   */

#include "Blinky.h"
#include "Mp3.h"
#include "Power.h"
#include "Display.h"

// Current active folder selected by Button Folder
int curFolder;

// Elapsed time
unsigned long idleTimer;


void setup() {

	// Waking up from power on
	//
  Serial.begin(115200);
  Serial.println();
  delay(20);
  Serial.println();
  Serial.println();
  Serial.println("BLINKY Version " + String(VERSION));
 
  // Init modules
  Power_init(); 
  delay(20);
  ButtonVolume_init();
  ButtonFolder_init();
  Display_init();
  Mp3_init();
 
  // Welcoming message
  Mp3_playFile(FOLDER_BLINKY,FILE_HELLO_ROBOT);

  // Initialisation of current folder
  curFolder = MP3_FIRST_FOLDER;
  Mp3_setFolder(curFolder);
  Display_show(curFolder, Power_getBatteryLevel());

  // Initialisation of idle timer with current time
  idleTimer = millis();

  Serial.println("MAIN ******** Initialisation completed **********");

}

void loop()
{
  //
  // 

  // When button Folder is turned, clockwise or anti-clockwise, 
  // - change the current MP3 folder
  // - display the folder image
  int steps = ButtonFolder_getTurnedNb();
  if (steps != 0) {
      Serial.println("Rotary encoder steps = " + String(steps));
      // Next folder index computed from button turn steps
      int newFolder = curFolder + steps;
      if (newFolder > MP3_LAST_FOLDER) {
        newFolder = MP3_FIRST_FOLDER;
      } else if (newFolder < MP3_FIRST_FOLDER) {
        newFolder = MP3_LAST_FOLDER;
      }
      // Change folder if different
      if (newFolder != curFolder) {
        curFolder = newFolder;
        Serial.println("New folder = " +String(curFolder));
        Mp3_stop();                 // Stop any on-going playback if any
        Mp3_setFolder(curFolder);   // and change folder
        int batteryLevel = Power_getBatteryLevel();         // Get battery level 0..100%
        Display_show(curFolder, batteryLevel);
      }
      // Reset idle timer
      idleTimer = millis();     
  }

  // When button Folder is pressed,
  // - play the next track within the current folder
  if (ButtonFolder_IsPushed()) {
    Serial.println("Button pushed, play next file");
    Mp3_playNextFile();
    // Reset idle timer
    idleTimer = millis();     
  }

  // When button Volume is turned
  // - change the volume accordingly
  steps = ButtonVolume_getTurnedNb();
  if (steps > 0) {    
    Mp3_volumeUp();
    // Reset idle timer
    idleTimer = millis();     
  } 
  else if (steps < 0) {
    Mp3_volumeDown();
    // Reset idle timer
    idleTimer = millis();     
  }

  /* Not used, to simplify 
  // When button Volume is pressed,
  // - pause or unpause
  if (ButtonVolume_IsPushed()) {
    Serial.print("Button volume pushed, ");
    if (Mp3_isBusy()) {
      // pause the current played file
      Serial.println("pause play");
      Mp3_pause();
    } else {
      // unpause the current file
      Serial.println("restart play");
      Mp3_start();
    }
   }
   */
  // If device is busy, reset the idle timer
  if (Mp3_isBusy())  {
    // Reset idle timer
    idleTimer = millis();         
  }

  // If the power switch has been pushed again and released, it's a request to shut down the system
  if (Power_offRequested()) {
    // Shut down
    Serial.println("Power off requested");
    Power_shutDown();       
  }
  
  // If nothing happened for the last IDLE_TIME_OUT minutes (no song played, no button interaction), shut down the system automatically
  if ((millis() - idleTimer) > IDLE_TIME_OUT) {
    // Shut down
    Serial.println("Idle time out, powering off");
    Power_shutDown();    
  }

  // Slow down a bit
  delay(200);
  
} 
